@@ -19,38 +19,6 @@ class TestViewer:
         viewer.load(table)
         assert viewer.table == table
 
-    # server-side view creation
-
-    def test_viewer_make_view(self):
-        table = Table({"a": [1, 2, 3]})
-        viewer = PerspectiveViewer(filters=[["a", "==", 2]])
-        viewer.load(table)
-        viewer._new_view()
-        assert viewer.view.to_dict() == {
-            "a": [2]
-        }
-
-    def test_viewer_make_view_none(self):
-        table = Table({"a": [1, 2, 3]})
-        viewer = PerspectiveViewer(filters=[["a", "==", 2]])
-        viewer.load(table)
-        assert viewer.view_name is None
-        assert viewer.view is None
-
-    def test_viewer_make_view_replace(self):
-        table = Table({"a": [1, 2, 3]})
-        viewer = PerspectiveViewer(filters=[["a", "==", 2]])
-        viewer.load(table)
-        viewer._new_view()
-        assert viewer.view.to_dict() == {
-            "a": [2]
-        }
-        viewer.filters = []
-        viewer._new_view()
-        assert viewer.view.to_dict() == {
-            "a": [1, 2, 3]
-        }
-
     # loading
 
     def test_viewer_load_table(self):
@@ -247,7 +215,7 @@ class TestViewer:
 
     def test_save_restore(self):
         table = Table({"a": [1, 2, 3]})
-        viewer = PerspectiveViewer(plugin="x_bar", filters=[["a", "==", 2]])
+        viewer = PerspectiveViewer(plugin="x_bar", filters=[["a", "==", 2]], editable=True)
         viewer.load(table)
 
         # Save config
@@ -256,13 +224,16 @@ class TestViewer:
         assert config["filters"] == [["a", "==", 2]]
         assert viewer.plugin == "x_bar"
         assert config["plugin"] == "x_bar"
+        assert config["editable"] is True
 
         # reset configuration
         viewer.reset()
         assert viewer.plugin == "datagrid"
         assert viewer.filters == []
+        assert viewer.editable is False
 
         # restore configuration
         viewer.restore(**config)
         assert viewer.filters == [["a", "==", 2]]
         assert viewer.plugin == "x_bar"
+        assert viewer.editable is True
